﻿using Microsoft.SharePoint.Client;
using ContactManagementWeb.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

namespace ContactManagementWeb.Controllers
{
    public class HomeController : Controller
    {
        [SharePointContextFilter]
        public ActionResult Index()
        {
            Uri hostWebUri = new Uri(Request.QueryString["SPHostUrl"]);
            List<Contact> contacts = new List<Contact>();

            var spContext = SharePointContextProvider.Current.GetSharePointContext(HttpContext);

            using (var ctx = spContext.CreateUserClientContextForSPHost())
            {
                List list = ctx.Web.Lists.GetByTitle("Contacts");
                ctx.Load(list);

                CamlQuery query = CamlQuery.CreateAllItemsQuery();
                
                ListItemCollection items = list.GetItems(query);
                ctx.Load(items);
                ctx.ExecuteQuery();

                foreach (ListItem item in items)
                {
                    contacts.Add(new Contact()
                    {
                        Id = item["ID"].ToString(),
                        LastName = item["Title"].ToString(),
                        FirstName = item["FirstName"].ToString(),
                        Email = item["Email"].ToString()
                    });
                }

            }

            ViewBag.SPHostUrl = Request.QueryString["SPHostUrl"];
            ViewBag.Contacts = contacts;
            return View();
        }

        public ActionResult EditContact(String SPHostUrl, String Id)
        {
            Contact contact = new Contact();
            Uri hostWebUri = new Uri(SPHostUrl);
            var spContext = SharePointContextProvider.Current.GetSharePointContext(HttpContext);

            using (var ctx = spContext.CreateUserClientContextForSPHost())
            {
                List list = ctx.Web.Lists.GetByTitle("Contacts");
                ctx.Load(list);

                ListItem item = list.GetItemById(Id);
                ctx.Load(item);

                ctx.ExecuteQuery();

                contact.Id = item["ID"].ToString();
                contact.LastName = item["Title"].ToString();
                contact.FirstName = item["FirstName"].ToString();
                contact.Email = item["Email"].ToString();

            }

            ViewBag.SPHostUrl = SPHostUrl;
            ViewBag.Contact = contact;
            return View();
        }

        public ActionResult UpdateContact(String SPHostUrl, String Id, String LastName, string FirstName, string Email)
        {
            Uri hostWebUri = new Uri(SPHostUrl);
            var spContext = SharePointContextProvider.Current.GetSharePointContext(HttpContext);

            using (var ctx = spContext.CreateUserClientContextForSPHost())
            {
                List list = ctx.Web.Lists.GetByTitle("Contacts");
                ctx.Load(list);

                ListItem item = list.GetItemById(Id);
                ctx.Load(item);

                ctx.ExecuteQuery();

                item["Title"] = LastName;
                item["FirstName"] = FirstName;
                item["Email"] = Email;
                item.Update();
                ctx.ExecuteQuery();


            }
            return RedirectToAction("Index", new { SPHostUrl = SPHostUrl });
        }
    }
}
